/****************************************************************************/
/*                                                                          */
/*  The FreeType project -- a free and portable quality TrueType renderer.  */
/*                                                                          */
/*  Copyright 1996, 1997 by                                                 */
/*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
/*                                                                          */
/*  ftdump.c : Simple TrueType font file resource profiler                  */
/*                                                                          */
/*  This program dumps various properties of a given font file              */
/*                                                                          */
/*                                                                          */
/*                                                                          */
/*  NOTE:  This is just a test program that is used to show off and         */
/*         debug the current engine; which is still in alpha.               */
/*                                                                          */
/****************************************************************************/

#ifdef ARM
#include "std.h"
#include "graflink.h"
#endif

#include "freetype.h"
#include "tterror.h"  /* for Panic() */

#include "ttobjs.h"   /* We're going to access internal tables directly */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef ARM
#include "armsup.c" /* pull in our routines */
#endif

  TT_Error     error;

  TT_Face      face;
  TT_Instance  instance;
  TT_Glyph     glyph;

  TT_Instance_Metrics  imetrics;
  TT_Glyph_Outline     outline;
  TT_Glyph_Metrics     metrics;

  TT_Face_Properties   properties;

  TT_CharMap    cmap;

  int  num_glyphs;
  int  ptsize;

  int  Fail;
  int  Num;


extern long TTMemory_Allocated;

#define MEMINFO(x)                                          \
           cur_memory = TTMemory_Allocated - old_memory;    \
           Message( "%6d bytes (%4d kb) :  %s\n",           \
                    cur_memory,                             \
                    (cur_memory+1023)/1024,                 \
                    x );                                    \
           old_memory += cur_memory;



  int  main( int  argc, char**  argv ) 
  {
    int    i;
    char   filename[128 + 4];
    char   alt_filename[128 + 4];
    char*  execname;

    long   org_memory, old_memory, cur_memory;
  
    execname = argv[0];

    if ( argc != 2 )
    {
      Message( "ftdump: simple TrueType Dumper - part of the FreeType project\n" );
      Message( "-------------------------------------------------------------\n\n" );
      Message( "Usage: %s fontname[.ttf|.ttc]\n\n", execname );
      exit( 1 );
    }

    i = strlen( argv[1] );
    while ( i > 0 && argv[1][i] != '\\' )
    {
      if ( argv[1][i] == '.' )
        i = 0;
      i--;
    }

    filename[128] = '\0';
    alt_filename[128] = '\0';

    strncpy( filename, argv[1], 128 );
    strncpy( alt_filename, argv[1], 128 );

    if ( i >= 0 )
    {
      strncpy( filename + strlen( filename ), ".ttf", 4 );
      strncpy( alt_filename + strlen( alt_filename ), ".ttc", 4 );
    }

    /* Initialize engine */

    old_memory = 0;

    if ( (error = TT_Init_FreeType()) )
      Panic( "ERROR: While initializing engine, code = %d\n", error );

    MEMINFO( "initial engine overhead" );

    org_memory = old_memory;

    /* Load face */

    error = TT_Open_Face( filename, &face );

    if (error == TT_Err_Could_Not_Open_File)
    {
      strcpy( filename, alt_filename );
      error = TT_Open_Face( alt_filename, &face );
    }

    if ( error )
      Panic( "ERROR: could not find/open %s\n", filename );

    MEMINFO( "face object creation" );

    /* get face properties and allocate preload arrays */

    TT_Get_Face_Properties( face, &properties );

    num_glyphs = properties.num_Glyphs;
        
    /* create glyph */

    error = TT_New_Glyph( face, &glyph );
    if ( error )
      Panic( "ERROR: could not create glyph container\n" );

    MEMINFO( "glyph object creation" );

    /* create instance */

    error = TT_New_Instance( face, &instance );
    if ( error )
      Panic( "ERROR: could not create instance for %s\n", filename );

    MEMINFO( "first instance object" );

    org_memory = TTMemory_Allocated - org_memory;

    error = TT_New_Instance( face, &instance );
    if ( error )
      Panic( "ERROR: could not create 2nd instance for %s\n", filename );

    MEMINFO( "each additional instance" );

    Message( "\nFont Memory Usage = %6d bytes (%4d kb)\n",
             (org_memory),
             (org_memory+1023)/1024 );

    error = TT_Set_Instance_PointSize( instance, ptsize );
    if ( error )
      Panic( "ERROR: could not reset instance for %s\n", filename );

    Fail = 0;

    TT_Close_Face( face );

    TT_Done_FreeType();

    return 0;
}


/* End */
