/****************************************************************************/
/*                                                                          */
/*  The FreeType project -- a free and portable quality TrueType renderer.  */
/*                                                                          */
/*  Copyright 1996, 1997 by                                                 */
/*  D. Turner, R.Wilhelm, and W. Lemberg                                    */
/*                                                                          */
/*  lint: a simple TrueType instruction tester.                             */
/*                                                                          */
/*  NOTE:  This is just a test program that is used to show off and         */
/*         debug the current engine; which is still in alpha.               */
/*                                                                          */
/****************************************************************************/

#ifdef ARM
#include "std.h"
#include "graflink.h"
#endif

#include "freetype.h"
#include "ttraster.h" /* The timer accessed the renderer directly */
#include "tterror.h"  /* for Panic() */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#ifdef ARM
#include "armsup.c" /* pull in our routines */
#endif

  TT_Error     error;

  TT_Face      face;
  TT_Instance  instance;
  TT_Glyph     glyph;

  TT_Glyph_Outline  outline;
  TT_Glyph_Metrics  metrics;

  TT_Face_Properties  properties;

  int  num_glyphs;
  int  ptsize;

  int  Fail;
  int  Num;


  static TT_Error  LoadTrueTypeChar( int  idx )
  {
    return TT_Load_Glyph( instance, glyph, idx, TTLOAD_DEFAULT );
  }


  int  main( int  argc, char**  argv ) 
  {
    int    i;
    char   filename[128 + 4];
    char   alt_filename[128 + 4];
    char*  execname;

  
    execname = argv[0];

    if ( argc != 3 )
    {
      Message( "test: simple TrueType interpreter tester - part of the FreeType project\n" );
      Message( "-----------------------------------------------------------------------\n\n" );
      Message( "Usage: %s ppem fontname[.ttf|.ttc]\n\n", execname );
      exit( 1 );
    }

    if ( sscanf( argv[1], "%d", &ptsize ) == EOF )
      ptsize = 64;

    i = strlen( argv[2] );
    while ( i > 0 && argv[2][i] != '\\' )
    {
      if ( argv[2][i] == '.' )
        i = 0;
      i--;
    }

    filename[128] = '\0';
    alt_filename[128] = '\0';

    strncpy( filename, argv[2], 128 );
    strncpy( alt_filename, argv[2], 128 );

    if ( i >= 0 )
    {
      strncpy( filename + strlen( filename ), ".ttf", 4 );
      strncpy( alt_filename + strlen( alt_filename ), ".ttc", 4 );
    }

    /* Initialize engine */

    if ( (error = TT_Init_FreeType()) )
      Panic( "ERROR: While initializing engine, code = %d\n", error );

    /* Load face */

    error = TT_Open_Face( filename, &face );

    if (error == TT_Err_Could_Not_Open_File)
    {
      strcpy( filename, alt_filename );
      error = TT_Open_Face( alt_filename, &face );
    }

    if ( error )
      Panic( "ERROR: could not find/open %s\n", filename );

    /* get face properties and allocate preload arrays */

    TT_Get_Face_Properties( face, &properties );

    num_glyphs = properties.num_Glyphs;
        
    /* create glyph */

    error = TT_New_Glyph( face, &glyph );
    if ( error )
      Panic( "ERROR: could not create glyph container\n" );

    /* create instance */

    error = TT_New_Instance( face, &instance );
    if ( error )
      Panic( "ERROR: could not create instance for %s\n", filename );

    error = TT_Set_Instance_PointSize( instance, ptsize );
    if ( error )
      Panic( "ERROR: could not reset instance for %s\n", filename );

    Fail = 0;

    Message( "Now executing glyphs\n" );

    for ( i = 0; i < num_glyphs; i++ )
    {
      if ( (error = LoadTrueTypeChar( i )) )
      {
        Message( "glyph %4d: error = %d\n", i, error );
        Fail++;
      }
    }

    TT_Done_FreeType();

    Message( "execution completed successfully\n" );
    Message( "fails  = %d\nglyphs = %d\n", Fail, num_glyphs );

    return 0;
}


/* End */
